/*
 * @author laurin
 * @date 09.08.2005
 * @version $Id: GoogleSearchExample.java,v 1.7 2009-12-16 16:12:02 herbert Exp $
 */
package com.gentics.portalnode.sdk;
import java.io.IOException;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;

import org.apache.axis.client.Service;

import com.gentics.api.lib.resolving.PropertySetter;
import com.gentics.api.portalnode.event.ActionEvent;
import com.gentics.api.portalnode.event.EventRequest;
import com.gentics.api.portalnode.event.EventResponse;
import com.gentics.api.portalnode.plugin.GenticsPlugin;
import com.gentics.api.portalnode.plugin.ViewPlugin;
import com.gentics.api.portalnode.portlet.AbstractGenticsPortlet;
import com.gentics.api.portalnode.portlet.GenticsPortletContext;
import com.gentics.api.portalnode.templateengine.TemplateProcessor;
import com.gentics.portalnode.genericmodules.examples.googlesearch.GoogleSearchBindingStub;
import com.gentics.portalnode.genericmodules.examples.googlesearch.GoogleSearchResult;

/**
 * sample google search, according to api.google.com. <br>the google wsdl
 * classes have been autogenerated from google's wsdl file using axis: <br>java -cp
 * mailapi.jar:activation.jar:saaj.jar:wsdl4j-1.5.1.jar:jaxrpc-1.2.1.jar:commons-discovery-0.2.jar:commons-logging-1.0.4.jar:axis-1.2.1.jar
 * org.apache.axis.wsdl.WSDL2Java -p
 * com.gentics.portalnode.genericmodules.examples.googlesearch
 * http://api.google.com/GoogleSearch.wsdl
 * @author laurin
 */
public class GoogleSearchExample extends AbstractGenticsPortlet {
    /**
     * view plugin
     */
    protected ViewPlugin viewPlugin;

    /**
     * @param moduleId
     * @throws PortletException
     */
    public GoogleSearchExample(String moduleId) throws PortletException {
        super(moduleId);
    }

    /**
     * create and register viewplugin
     */
    protected void onLoad() {
        super.onLoad();
        // create the views plugin
        GenticsPlugin plugin = getGenticsPortletContext().createPlugin("ViewPlugin",
                "viewplugin", getPortletConfig());
        if (plugin instanceof ViewPlugin) {
            viewPlugin = (ViewPlugin) plugin;
            registerPlugin("viewplugin", viewPlugin);
        }
    }

    /**
     * put view and resultlist into templateprocessor and render it.
     */
    protected void doView(RenderRequest request, RenderResponse response)
            throws PortletException, IOException {
        if (WindowState.NORMAL.equals(request.getWindowState())) {
            if (viewPlugin != null) {
                response.getWriter().print(renderPlugin("viewplugin", request, response));
            }
        } else if (WindowState.MAXIMIZED.equals(request.getWindowState())) {
            try {
                GenticsPortletContext ctx = getGenticsPortletContext();
                TemplateProcessor templateProcessor  = ctx.getTemplateProcessor(this, getPortletConfig());
                                
                templateProcessor.put("views", renderPlugin("viewplugin", request, response));
                
                Object result = request.getPortletSession().getAttribute("result");
                if (result instanceof GoogleSearchResult) {
                    templateProcessor.put("result", (GoogleSearchResult) result);
                }
                
                response.getWriter().print(templateProcessor.getOutput("main", this));
                returnTemplateProcessor(templateProcessor);
            } catch (Exception ex) {
                logger.error("error while rendering output", ex);
            }
        }
    }

    /**
     * listen for pluginevents for search button
     */
    public void onPluginEvent(EventRequest eventRequest, EventResponse eventResponse) {
        ActionEvent wrapperEvent = eventRequest.getEvent();
        ActionEvent event = wrapperEvent.getEmbeddedEvent();

        // wait for correct event
        if ("search".equals(event.getActionCommand())) {
            try {
                boolean doQuery = true;
                // prepare googlse wsdl api
                Service service = new Service();
                java.net.URL url = new java.net.URL("http://api.google.com/search/beta2");
                GoogleSearchBindingStub gs = new GoogleSearchBindingStub(url, service);
                
                // validate user input data, query string and google license.
                Object q = event.getParameter("query");
                if (q == null || "".equals(q)) {
                    logger.error("query was empty.");
                    doQuery = false;
                }
                
                Object lic = event.getParameter("license");
                if (lic == null || lic.equals("")) {
                    logger.error("license key was empty.");
                    doQuery = false;
                }
                
                // do the query and remember to result
                if(doQuery == true){
                    GoogleSearchResult result = gs.doGoogleSearch(lic.toString(), q.toString(), 0,
                        10, true, "", true, "", "", "");
                    eventRequest.getPortletSession().setAttribute("result", result);
                }
            } catch (IOException e) {
                PropertySetter setter = new PropertySetter(viewPlugin);
                try {
                    setter.setProperty("views.GoogleSearch.components.query.error",e.getLocalizedMessage());
                } catch (Exception ex) {
                    logger.error("failed to set error in view",ex);
                }
            }

        }

    }
}