/*
 * @author stefan
 * @date 16.08.2005
 * @version $Id: GenticsCalculatorModule.java,v 1.10 2006-01-13 17:50:12 laurin Exp $
 */

package com.gentics.portalnode.sdk.calculator;

import com.gentics.api.lib.exception.InsufficientPrivilegesException;
import com.gentics.api.lib.exception.UnknownPropertyException;
import com.gentics.api.lib.resolving.PropertyResolver;
import com.gentics.api.lib.resolving.PropertySetter;
import com.gentics.api.portalnode.event.ActionEvent;
import com.gentics.api.portalnode.event.EventRequest;
import com.gentics.api.portalnode.event.EventResponse;
import com.gentics.api.portalnode.plugin.GenticsPlugin;
import com.gentics.api.portalnode.plugin.ViewPlugin;
import com.gentics.api.portalnode.portlet.AbstractGenticsPortlet;
import com.gentics.portalnode.sdk.GenticsHistoryModule;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.PortletSession;
import java.io.IOException;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Collection;

/**
 * A simple demo calculator. A view is used to display a simple calculator, the
 * input fields can be processed by a pluginAction. The view must then be named
 * 'calc', the first input field must be 'input1', the second field must be
 * named 'input2'. The result will be written to the field 'output'.
 */
public class GenticsCalculatorModule extends AbstractGenticsPortlet {

    public GenticsCalculatorModule(String moduleId) throws PortletException {
        super(moduleId);
    }

    protected void onLoad() {
        super.onLoad();

        // create a new viewplugin
        GenticsPlugin plugin = getGenticsPortletContext().createPlugin(
                "ViewPlugin", "viewplugin", getPortletConfig());

        // register the plugin
        if (plugin instanceof ViewPlugin) {
            registerPlugin("viewplugin", plugin);
        }
    }

    protected void doView(RenderRequest renderRequest, RenderResponse renderResponse)
            throws PortletException, IOException {
        // render the viewplugin directly to the output
        renderResponse.getWriter().print(
                renderPlugin("viewplugin", renderRequest, renderResponse));
    }

    public void onPluginEvent(EventRequest eventRequest, EventResponse eventResponse) {
        super.onPluginEvent(eventRequest, eventResponse);

        // get the event which caused the pluginEvent
        ActionEvent event = eventRequest.getEvent().getEmbeddedEvent();

        // check if the original event was a 'calc'-event
        if ("calc".equals(event.getActionCommand())) {

            int val1 = 0;
            int val2 = 0;
            Object value;

            // get a resolver and a setter for the viewplugin
            PropertyResolver resolver = new PropertyResolver(getPlugin("viewplugin"));
            PropertySetter setter = new PropertySetter(getPlugin("viewplugin"));

            // alternativly, one could get the values of the fields by accessing
            // the view and its components directly:
            //            FormPlugin2 form = ((ViewPlugin)
            // getPlugin("viewplugin")).getFormPlugin2();
            //            String value1 = ((TextComponent)
            // form.getComponentById("input1")).getText();
            //            String value2 = ((TextComponent)
            // form.getComponentById("input2")).getText();

            try {

                // get the value from the first input field
                value = resolver.resolve("views.calc.components.input1.data");
                if (value != null) {
                    val1 = Integer.parseInt(value.toString());
                }

                // get the value from the second input field
                value = resolver.resolve("views.calc.components.input2.data");
                if (value != null) {
                    val2 = Integer.parseInt(value.toString());
                }

                // get the operand from the event parameters
                int result = 0;
                String operand = event.getParameter("operand").toString();

                // calculate the result
                if ("+".equals(operand)) {
                    result = val1 + val2;
                } else if ("-".equals(operand)) {
                    result = val1 - val2;
                } else if ("*".equals(operand)) {
                    result = val1 * val2;
                } else if ("/".equals(operand)) {
                    result = val1 / val2;
                }

                // set the result into the output-field
                setter.setProperty("views.calc.components.output.data", Integer
                        .toString(result));

                

                // store the result in the session-attribute for the history module
                Object data = eventRequest.getPortletSession().getAttribute(
                        GenticsHistoryModule.SESSION_DATA, PortletSession.APPLICATION_SCOPE);

                // create a new list for the session
                if (data == null) {
                    data = Collections.synchronizedList(new ArrayList());
                    eventRequest.getPortletSession().setAttribute(
                            GenticsHistoryModule.SESSION_DATA, data,
                            PortletSession.APPLICATION_SCOPE);
                }

                // add the operation-string to the history-module by session-attribute
                if (data instanceof Collection) {
                    ((Collection) data).add(val1 + " " + operand + " " + val2 + " = " + result);
                }


            } catch (NumberFormatException e) {
                try {
                    // if a number could not be parsed, add an errormessage to
                    // the view
                    setter.setProperty("views.calc.components.output.error", "Invalid number");
                } catch (Exception e1) {
                    // write error log, if the error message could not be set in
                    // the view.
                    logger.error("error message could not be set to view", e);
                }
            } catch (ArithmeticException e) {
                try {
                    // if an operation throws an exception, add an errormessage to
                    // the view
                    setter.setProperty("views.calc.components.output.error", "Invalid operation");
                } catch (Exception e1) {
                    // write error log, if the error message could not be set in
                    // the view.
                    logger.error("error message could not be set to view", e);
                }

                // catch exceptions during resolving or setting of view values.
            } catch (UnknownPropertyException e) {
                logger.error(e);
            } catch (InsufficientPrivilegesException e) {
                logger.error(e);
            }
        }
    }
}
