package com.gentics.portalnode.sdk.filter;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import com.gentics.api.lib.datasource.Datasource;
import com.gentics.api.lib.expressionparser.ExpressionParser;
import com.gentics.api.lib.resolving.Resolvable;
import com.gentics.api.portalnode.portlet.AbstractGenticsPortlet;

/**
 * Example Portlet for usage of the filter() function with PostProcessors
 */
public class PostProcessorExample extends AbstractGenticsPortlet {
	/**
	 * Create an instance
	 * @param moduleId portlet id
	 * @throws PortletException
	 */
	public PostProcessorExample(String moduleId) throws PortletException {
		super(moduleId);
	}

	/* (non-Javadoc)
	 * @see javax.portlet.GenericPortlet#doView(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
	 */
	@Override
	protected void doView(RenderRequest request, RenderResponse response)
			throws PortletException, IOException {
		// simple rule (without filter)
		getResultForFilter("Normal rule", "object.obj_type == 1001", request, response);
		// single filter that removes objects
		getResultForFilter("Restricting PostProcessor", "filter(object.obj_type == 1001, 'com.gentics.portalnode.sdk.filter.StartsWith', ['firstname', 'B'])", request, response);
		// sorting filter
		getResultForFilter("Sorting PostProcessor", "filter(object.obj_type == 1001, 'com.gentics.portalnode.sdk.filter.Sorter', 'surname')", request, response);
		// combine two filters
		getResultForFilter("Multiple PostProcessors", "filter(filter(object.obj_type == 1001, 'com.gentics.portalnode.sdk.filter.StartsWith', ['firstname', 'B']), 'com.gentics.portalnode.sdk.filter.Sorter', 'surname')", request, response);
	}

	/**
	 * Render the objects filtered by the given filter
	 * @param description description
	 * @param filter filter
	 * @param request render request
	 * @param response render response
	 * @throws PortletException
	 * @throws IOException
	 */
	protected void getResultForFilter(String description, String filter, RenderRequest request,
			RenderResponse response) throws PortletException, IOException {
		try {
			Datasource ds = getGenticsPortletContext().getDatasource();
			Collection persons = ds.getResult(ds
					.createDatasourceFilter(ExpressionParser.getInstance()
							.parse(filter)), null);
			PrintWriter writer = response.getWriter();
			writer.println("<h1>" + description + "</h1>");
			writer.println("<b>" + filter + "</b><br/>");
			for (Object o : persons) {
				if (o instanceof Resolvable) {
					Resolvable person = (Resolvable) o;
					writer.println(person.get("firstname") + " "
							+ person.get("surname") + "<br/>");
				}
			}
		} catch (Exception e) {
			throw new PortletException(e);
		}
	}
}
